@@ -16,7 +16,6 @@ use std::time::SystemTime;
 
 mod http;
 mod lsp;
-mod throughput;
 
 fn read_json(filename: &str) -> Result<Value> {
   let f = fs::File::open(filename)?;
@@ -326,17 +325,6 @@ fn bundle_benchmark(deno_exe: &Path) -> Result<HashMap<String, u64>> {
   Ok(sizes)
 }
 
-fn run_throughput(deno_exe: &Path) -> Result<HashMap<String, f64>> {
-  let mut m = HashMap::<String, f64>::new();
-
-  m.insert("100M_tcp".to_string(), throughput::tcp(deno_exe, 100)?);
-  m.insert("100M_cat".to_string(), throughput::cat(deno_exe, 100));
-  m.insert("10M_tcp".to_string(), throughput::tcp(deno_exe, 10)?);
-  m.insert("10M_cat".to_string(), throughput::cat(deno_exe, 10));
-
-  Ok(m)
-}
-
 fn run_http(target_dir: &Path, new_data: &mut BenchResult) -> Result<()> {
   let stats = http::benchmark(target_dir)?;
 
@@ -452,7 +440,6 @@ struct BenchResult {
   req_per_sec: HashMap<String, u64>,
   syscall_count: HashMap<String, u64>,
   thread_count: HashMap<String, u64>,
-  throughput: HashMap<String, f64>,
 }
 
 /*
@@ -495,10 +482,7 @@ fn main() -> Result<()> {
     ..Default::default()
   };
 
-  // Cannot run throughput benchmark on windows because they don't have nc or
-  // pipe.
   if cfg!(not(target_os = "windows")) {
-    new_data.throughput = run_throughput(&deno_exe)?;
     run_http(&target_dir, &mut new_data)?;
   }
 
