@@ -29,6 +29,7 @@ use super::analysis::ts_changes_to_edit;
 use super::analysis::CodeActionCollection;
 use super::analysis::CodeActionData;
 use super::analysis::ResolvedDependency;
+use super::cache::CacheServer;
 use super::capabilities;
 use super::code_lens;
 use super::completions;
@@ -44,7 +45,6 @@ use super::parent_process_checker;
 use super::performance::Performance;
 use super::refactor;
 use super::registries;
-use super::sources;
 use super::sources::Sources;
 use super::text;
 use super::text::LineIndex;
@@ -54,6 +54,8 @@ use super::tsc::Assets;
 use super::tsc::TsServer;
 use super::urls;
 use crate::config_file::ConfigFile;
+use crate::config_file::FmtConfig;
+use crate::config_file::LintConfig;
 use crate::config_file::TsConfig;
 use crate::deno_dir;
 use crate::file_fetcher::get_source_from_data_url;
@@ -73,6 +75,8 @@ pub struct StateSnapshot {
   pub assets: Assets,
   pub config: ConfigSnapshot,
   pub documents: DocumentCache,
+  pub maybe_lint_config: Option<LintConfig>,
+  pub maybe_fmt_config: Option<FmtConfig>,
   pub maybe_config_uri: Option<ModuleSpecifier>,
   pub module_registries: registries::ModuleRegistry,
   pub performance: Performance,
@@ -99,9 +103,15 @@ pub(crate) struct Inner {
   /// An optional path to the DENO_DIR which has been specified in the client
   /// options.
   maybe_cache_path: Option<PathBuf>,
+  /// A lazily created "server" for handling cache requests.
+  maybe_cache_server: Option<CacheServer>,
   /// An optional configuration file which has been specified in the client
   /// options.
   maybe_config_file: Option<ConfigFile>,
+  /// An optional configuration for linter which has been taken from specified config file.
+  maybe_lint_config: Option<LintConfig>,
+  /// An optional configuration for formatter which has been taken from specified config file.
+  maybe_fmt_config: Option<FmtConfig>,
   /// An optional URL which provides the location of a TypeScript configuration
   /// file which will be used by the Deno LSP.
   maybe_config_uri: Option<Url>,
@@ -149,6 +159,9 @@ impl Inner {
       diagnostics_server,
       documents: Default::default(),
       maybe_cache_path: None,
+      maybe_lint_config: None,
+      maybe_fmt_config: None,
+      maybe_cache_server: None,
       maybe_config_file: None,
       maybe_config_uri: None,
       maybe_import_map: None,
@@ -385,16 +398,9 @@ impl Inner {
     &mut self,
     tsconfig: &mut TsConfig,
   ) -> Result<(), AnyError> {
-    self.maybe_config_file = None;
-    self.maybe_config_uri = None;
-
-    let maybe_file_and_url = self.get_config_file_and_url()?;
-
-    if let Some((config_file, config_url)) = maybe_file_and_url {
+    if let Some(config_file) = self.maybe_config_file.as_ref() {
       let (value, maybe_ignored_options) = config_file.to_compiler_options()?;
       tsconfig.merge(&value);
-      self.maybe_config_file = Some(config_file);
-      self.maybe_config_uri = Some(config_url);
       if let Some(ignored_options) = maybe_ignored_options {
         // TODO(@kitsonk) turn these into diagnostics that can be sent to the
         // client
@@ -413,6 +419,8 @@ impl Inner {
         LspError::internal_error()
       })?,
       documents: self.documents.clone(),
+      maybe_lint_config: self.maybe_lint_config.clone(),
+      maybe_fmt_config: self.maybe_fmt_config.clone(),
       maybe_config_uri: self.maybe_config_uri.clone(),
       module_registries: self.module_registries.clone(),
       performance: self.performance.clone(),
@@ -424,6 +432,7 @@ impl Inner {
   pub fn update_cache(&mut self) -> Result<(), AnyError> {
     let mark = self.performance.mark("update_cache", None::<()>);
     self.performance.measure(mark);
+    self.maybe_cache_server = None;
     let (maybe_cache, maybe_root_uri) = {
       let config = &self.config;
       (
@@ -479,6 +488,7 @@ impl Inner {
 
   pub async fn update_import_map(&mut self) -> Result<(), AnyError> {
     let mark = self.performance.mark("update_import_map", None::<()>);
+    self.maybe_cache_server = None;
     let (maybe_import_map, maybe_root_uri) = {
       let config = &self.config;
       (
@@ -574,6 +584,37 @@ impl Inner {
     Ok(())
   }
 
+  fn update_config_file(&mut self) -> Result<(), AnyError> {
+    self.maybe_config_file = None;
+    self.maybe_config_uri = None;
+    self.maybe_fmt_config = None;
+    self.maybe_lint_config = None;
+
+    let maybe_file_and_url = self.get_config_file_and_url()?;
+
+    if let Some((config_file, config_url)) = maybe_file_and_url {
+      let lint_config = config_file
+        .to_lint_config()
+        .map_err(|err| {
+          anyhow!("Unable to update lint configuration: {:?}", err)
+        })?
+        .unwrap_or_default();
+      let fmt_config = config_file
+        .to_fmt_config()
+        .map_err(|err| {
+          anyhow!("Unable to update formatter configuration: {:?}", err)
+        })?
+        .unwrap_or_default();
+
+      self.maybe_config_file = Some(config_file);
+      self.maybe_config_uri = Some(config_url);
+      self.maybe_lint_config = Some(lint_config);
+      self.maybe_fmt_config = Some(fmt_config);
+    }
+
+    Ok(())
+  }
+
   async fn update_tsconfig(&mut self) -> Result<(), AnyError> {
     let mark = self.performance.mark("update_tsconfig", None::<()>);
     let mut tsconfig = TsConfig::new(json!({
@@ -689,6 +730,9 @@ impl Inner {
     if let Err(err) = self.update_cache() {
       self.client.show_message(MessageType::Warning, err).await;
     }
+    if let Err(err) = self.update_config_file() {
+      self.client.show_message(MessageType::Warning, err).await;
+    }
     if let Err(err) = self.update_tsconfig().await {
       self.client.show_message(MessageType::Warning, err).await;
     }
@@ -913,6 +957,9 @@ impl Inner {
     if let Err(err) = self.update_registries().await {
       self.client.show_message(MessageType::Warning, err).await;
     }
+    if let Err(err) = self.update_config_file() {
+      self.client.show_message(MessageType::Warning, err).await;
+    }
     if let Err(err) = self.update_tsconfig().await {
       self.client.show_message(MessageType::Warning, err).await;
     }
@@ -943,6 +990,9 @@ impl Inner {
     // if the current tsconfig has changed, we need to reload it
     if let Some(config_uri) = &self.maybe_config_uri {
       if params.changes.iter().any(|fe| *config_uri == fe.uri) {
+        if let Err(err) = self.update_config_file() {
+          self.client.show_message(MessageType::Warning, err).await;
+        }
         if let Err(err) = self.update_tsconfig().await {
           self.client.show_message(MessageType::Warning, err).await;
         }
@@ -1026,19 +1076,8 @@ impl Inner {
         PathBuf::from(params.text_document.uri.path())
       };
 
-    let maybe_file_and_url = self.get_config_file_and_url().map_err(|err| {
-      error!("Unable to parse configuration file: {}", err);
-      LspError::internal_error()
-    })?;
-
-    let fmt_options = if let Some((config_file, _)) = maybe_file_and_url {
-      config_file
-        .to_fmt_config()
-        .map_err(|err| {
-          error!("Unable to parse fmt configuration: {}", err);
-          LspError::internal_error()
-        })?
-        .unwrap_or_default()
+    let fmt_options = if let Some(fmt_config) = self.maybe_fmt_config.as_ref() {
+      fmt_config.options.clone()
     } else {
       Default::default()
     };
@@ -1047,16 +1086,12 @@ impl Inner {
     let text_edits = tokio::task::spawn_blocking(move || {
       let format_result = match source.module() {
         Some(Ok(parsed_module)) => {
-          Ok(format_parsed_module(parsed_module, fmt_options.options))
+          Ok(format_parsed_module(parsed_module, fmt_options))
         }
         Some(Err(err)) => Err(err.to_string()),
         None => {
           // it's not a js/ts file, so attempt to format its contents
-          format_file(
-            &file_path,
-            source.text_info().text_str(),
-            fmt_options.options,
-          )
+          format_file(&file_path, source.text_info().text_str(), fmt_options)
         }
       };
 
@@ -2630,34 +2665,30 @@ impl Inner {
     }
 
     let mark = self.performance.mark("cache", Some(&params));
-    if !params.uris.is_empty() {
-      for identifier in &params.uris {
-        let specifier = self.url_map.normalize_url(&identifier.uri);
-        sources::cache(
-          &specifier,
-          &self.maybe_import_map,
-          &self.maybe_config_file,
-          &self.maybe_cache_path,
-        )
-        .await
-        .map_err(|err| {
-          error!("{}", err);
-          LspError::internal_error()
-        })?;
-      }
+    let roots = if !params.uris.is_empty() {
+      params
+        .uris
+        .iter()
+        .map(|t| self.url_map.normalize_url(&t.uri))
+        .collect()
     } else {
-      sources::cache(
-        &referrer,
-        &self.maybe_import_map,
-        &self.maybe_config_file,
-        &self.maybe_cache_path,
-      )
-      .await
-      .map_err(|err| {
-        error!("{}", err);
-        LspError::internal_error()
-      })?;
+      vec![referrer.clone()]
+    };
+
+    if self.maybe_cache_server.is_none() {
+      self.maybe_cache_server = Some(
+        CacheServer::new(
+          self.maybe_cache_path.clone(),
+          self.maybe_import_map.clone(),
+        )
+        .await,
+      );
     }
+    let cache_server = self.maybe_cache_server.as_ref().unwrap();
+    if let Err(err) = cache_server.cache(roots).await {
+      self.client.show_message(MessageType::Warning, err).await;
+    }
+
     // now that we have dependencies loaded, we need to re-analyze them and
     // invalidate some diagnostics
     if self.documents.contains_key(&referrer) {
