@@ -856,11 +856,18 @@ itest!(wasm_async {
 });
 
 itest!(wasm_unreachable {
-  args: "run wasm_unreachable.js",
+  args: "run --allow-read wasm_unreachable.js",
   output: "wasm_unreachable.out",
   exit_code: 1,
 });
 
+itest!(wasm_url {
+  args: "run --quiet --allow-net=localhost:4545 wasm_url.js",
+  output: "wasm_url.out",
+  exit_code: 1,
+  http_server: true,
+});
+
 itest!(weakref {
   args: "run --quiet --reload weakref.ts",
   output: "weakref.ts.out",
@@ -1178,6 +1185,13 @@ itest!(error_import_map_unable_to_load {
   exit_code: 1,
 });
 
+// Test that setting `self` in the main thread to some other value doesn't break
+// the world.
+itest!(replace_self {
+  args: "run replace_self.js",
+  output: "replace_self.js.out",
+});
+
 itest!(worker_event_handler_test {
   args: "run --quiet --reload --allow-read worker_event_handler_test.js",
   output: "worker_event_handler_test.js.out",
@@ -1204,6 +1218,11 @@ itest!(worker_message_before_close {
   output: "worker_message_before_close.js.out",
 });
 
+itest!(worker_close_in_wasm_reactions {
+  args: "run --quiet --reload --allow-read worker_close_in_wasm_reactions.js",
+  output: "worker_close_in_wasm_reactions.js.out",
+});
+
 #[test]
 fn no_validate_asm() {
   let output = util::deno_cmd()
