@@ -1,14 +1,14 @@
 // Copyright 2018-2021 the Deno authors. All rights reserved. MIT license.
 import {
   assertEquals,
+  assertRejects,
   assertThrows,
-  assertThrowsAsync,
   unitTest,
 } from "./test_util.ts";
 
 unitTest(
-  { perms: { read: true, write: true } },
-  function writeFileSyncSuccess(): void {
+  { permissions: { read: true, write: true } },
+  function writeFileSyncSuccess() {
     const enc = new TextEncoder();
     const data = enc.encode("Hello");
     const filename = Deno.makeTempDirSync() + "/test.txt";
@@ -21,8 +21,8 @@ unitTest(
 );
 
 unitTest(
-  { perms: { read: true, write: true } },
-  function writeFileSyncUrl(): void {
+  { permissions: { read: true, write: true } },
+  function writeFileSyncUrl() {
     const enc = new TextEncoder();
     const data = enc.encode("Hello");
     const tempDir = Deno.makeTempDirSync();
@@ -39,7 +39,7 @@ unitTest(
   },
 );
 
-unitTest({ perms: { write: true } }, function writeFileSyncFail(): void {
+unitTest({ permissions: { write: true } }, function writeFileSyncFail() {
   const enc = new TextEncoder();
   const data = enc.encode("Hello");
   const filename = "/baddir/test.txt";
@@ -49,7 +49,7 @@ unitTest({ perms: { write: true } }, function writeFileSyncFail(): void {
   }, Deno.errors.NotFound);
 });
 
-unitTest({ perms: { write: false } }, function writeFileSyncPerm(): void {
+unitTest({ permissions: { write: false } }, function writeFileSyncPerm() {
   const enc = new TextEncoder();
   const data = enc.encode("Hello");
   const filename = "/baddir/test.txt";
@@ -60,8 +60,8 @@ unitTest({ perms: { write: false } }, function writeFileSyncPerm(): void {
 });
 
 unitTest(
-  { perms: { read: true, write: true } },
-  function writeFileSyncUpdateMode(): void {
+  { permissions: { read: true, write: true } },
+  function writeFileSyncUpdateMode() {
     if (Deno.build.os !== "windows") {
       const enc = new TextEncoder();
       const data = enc.encode("Hello");
@@ -75,8 +75,8 @@ unitTest(
 );
 
 unitTest(
-  { perms: { read: true, write: true } },
-  function writeFileSyncCreate(): void {
+  { permissions: { read: true, write: true } },
+  function writeFileSyncCreate() {
     const enc = new TextEncoder();
     const data = enc.encode("Hello");
     const filename = Deno.makeTempDirSync() + "/test.txt";
@@ -96,8 +96,8 @@ unitTest(
 );
 
 unitTest(
-  { perms: { read: true, write: true } },
-  function writeFileSyncAppend(): void {
+  { permissions: { read: true, write: true } },
+  function writeFileSyncAppend() {
     const enc = new TextEncoder();
     const data = enc.encode("Hello");
     const filename = Deno.makeTempDirSync() + "/test.txt";
@@ -121,8 +121,8 @@ unitTest(
 );
 
 unitTest(
-  { perms: { read: true, write: true } },
-  async function writeFileSuccess(): Promise<void> {
+  { permissions: { read: true, write: true } },
+  async function writeFileSuccess() {
     const enc = new TextEncoder();
     const data = enc.encode("Hello");
     const filename = Deno.makeTempDirSync() + "/test.txt";
@@ -135,8 +135,8 @@ unitTest(
 );
 
 unitTest(
-  { perms: { read: true, write: true } },
-  async function writeFileUrl(): Promise<void> {
+  { permissions: { read: true, write: true } },
+  async function writeFileUrl() {
     const enc = new TextEncoder();
     const data = enc.encode("Hello");
     const tempDir = await Deno.makeTempDir();
@@ -154,34 +154,34 @@ unitTest(
 );
 
 unitTest(
-  { perms: { read: true, write: true } },
-  async function writeFileNotFound(): Promise<void> {
+  { permissions: { read: true, write: true } },
+  async function writeFileNotFound() {
     const enc = new TextEncoder();
     const data = enc.encode("Hello");
     const filename = "/baddir/test.txt";
     // The following should fail because /baddir doesn't exist (hopefully).
-    await assertThrowsAsync(async () => {
+    await assertRejects(async () => {
       await Deno.writeFile(filename, data);
     }, Deno.errors.NotFound);
   },
 );
 
 unitTest(
-  { perms: { read: true, write: false } },
-  async function writeFilePerm(): Promise<void> {
+  { permissions: { read: true, write: false } },
+  async function writeFilePerm() {
     const enc = new TextEncoder();
     const data = enc.encode("Hello");
     const filename = "/baddir/test.txt";
     // The following should fail due to no write permission
-    await assertThrowsAsync(async () => {
+    await assertRejects(async () => {
       await Deno.writeFile(filename, data);
     }, Deno.errors.PermissionDenied);
   },
 );
 
 unitTest(
-  { perms: { read: true, write: true } },
-  async function writeFileUpdateMode(): Promise<void> {
+  { permissions: { read: true, write: true } },
+  async function writeFileUpdateMode() {
     if (Deno.build.os !== "windows") {
       const enc = new TextEncoder();
       const data = enc.encode("Hello");
@@ -195,13 +195,13 @@ unitTest(
 );
 
 unitTest(
-  { perms: { read: true, write: true } },
-  async function writeFileCreate(): Promise<void> {
+  { permissions: { read: true, write: true } },
+  async function writeFileCreate() {
     const enc = new TextEncoder();
     const data = enc.encode("Hello");
     const filename = Deno.makeTempDirSync() + "/test.txt";
     // if create turned off, the file won't be created
-    await assertThrowsAsync(async () => {
+    await assertRejects(async () => {
       await Deno.writeFile(filename, data, { create: false });
     }, Deno.errors.NotFound);
 
@@ -216,8 +216,8 @@ unitTest(
 );
 
 unitTest(
-  { perms: { read: true, write: true } },
-  async function writeFileAppend(): Promise<void> {
+  { permissions: { read: true, write: true } },
+  async function writeFileAppend() {
     const enc = new TextEncoder();
     const data = enc.encode("Hello");
     const filename = Deno.makeTempDirSync() + "/test.txt";
@@ -239,3 +239,39 @@ unitTest(
     assertEquals("Hello", actual);
   },
 );
+
+unitTest(
+  { permissions: { read: true, write: true } },
+  async function writeFileAbortSignal(): Promise<void> {
+    const ac = new AbortController();
+    const enc = new TextEncoder();
+    const data = enc.encode("Hello");
+    const filename = Deno.makeTempDirSync() + "/test.txt";
+    queueMicrotask(() => ac.abort());
+    try {
+      await Deno.writeFile(filename, data, { signal: ac.signal });
+    } catch (e) {
+      assertEquals(e.name, "AbortError");
+    }
+    const stat = Deno.statSync(filename);
+    assertEquals(stat.size, 0);
+  },
+);
+
+unitTest(
+  { permissions: { read: true, write: true } },
+  async function writeFileAbortSignalPreAborted(): Promise<void> {
+    const ac = new AbortController();
+    ac.abort();
+    const enc = new TextEncoder();
+    const data = enc.encode("Hello");
+    const filename = Deno.makeTempDirSync() + "/test.txt";
+    try {
+      await Deno.writeFile(filename, data, { signal: ac.signal });
+    } catch (e) {
+      assertEquals(e.name, "AbortError");
+    }
+    const stat = Deno.statSync(filename);
+    assertEquals(stat.size, 0);
+  },
+);
