@@ -485,7 +485,7 @@ impl ReplSession {
 
   pub async fn is_closing(&mut self) -> Result<bool, AnyError> {
     let closed = self
-      .evaluate_expression("(globalThis.closed)")
+      .evaluate_expression("(this.closed)")
       .await?
       .get("result")
       .unwrap()
@@ -655,6 +655,7 @@ impl ReplSession {
       media_type: deno_ast::MediaType::TypeScript,
       capture_tokens: false,
       maybe_syntax: None,
+      scope_analysis: false,
     })?;
 
     let transpiled_src = transpile(
